#!/usr/bin/env python
# -*- coding: utf-8 -*-
# $Id: start_project.py 13367 2021-12-24 04:53:31Z Mint $
#
# Copyright (c) 2016 Nuwa Information Co., Ltd, All Rights Reserved.
#
# Licensed under the Proprietary License,
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at our web site.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# $Author: Mint $
# $Date: 2021-12-24 12:53:31 +0800 (週五, 24 十二月 2021) $
# $Revision: 13367 $

import os
import sys
import six
import shutil
import datetime
#import django

from django.core.management.base import BaseCommand, CommandError #pylint: disable=W0611
from django.template.loader import get_template
from django.template import Context #pylint: disable=W0611

from Iuno.Version import IUNO_VERSION
from Iuppiter.Encoding import utf8

# Mercurius
IUNO_BASE_PATH = "\\Mercurius\\trunk\\Iuno\\Iuno"

# Pandora
SCAFFOLD_TEMP_PATH = "\\Pandora\\trunk\\Containers\\Apps\\Scaffold\\_Scaffold"
SCAFFOLD_PATH = "\\Pandora\\trunk\\Containers\\Apps\\Scaffold\\Scaffold"

# http://landcareweb.com/questions/6325/python-yi-dong-bing-fu-gai-wen-jian-he-wen-jian-jia
def forceCopyFile(sfile, dfile):
    if os.path.isfile(sfile):
        shutil.copy2(sfile, dfile)
        
def forceMergeFlatDir(srcDir, dstDir):
    if not os.path.exists(dstDir):
        os.makedirs(dstDir)
    for item in os.listdir(srcDir):
        srcFile = os.path.join(srcDir, item)
        dstFile = os.path.join(dstDir, item)
        forceCopyFile(srcFile, dstFile)
        
def isAFlatDir(sDir):
    for item in os.listdir(sDir):
        sItem = os.path.join(sDir, item)
        if os.path.isdir(sItem):
            return False
    return True

def copyTree(src, dst):
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isfile(s):
            if not os.path.exists(dst):
                os.makedirs(dst)
            forceCopyFile(s, d)
        if os.path.isdir(s):
            isRecursive = not isAFlatDir(s)
            if isRecursive:
                copyTree(s, d)
            else:
                forceMergeFlatDir(s, d)


class Command(BaseCommand):
    help = 'Build django project with our setting and files.'

    def add_arguments(self, parser):
        parser.add_argument('name', type=str, nargs='?', help="Project name")
        parser.add_argument('path', type=str, nargs='?', help="Project path")
        parser.add_argument(
            '--pandora', 
            action='store_true',
            help="Update or create Pandora Scaffold project.")

    def handle(self, *args, **options):
        path = options['path']
        name = options['name']
        pandora = options['pandora']
        
        if pandora:
            name = "Scaffold"
            path = os.path.abspath(os.path.dirname(__name__))
            pandoraPath = path.replace(IUNO_BASE_PATH, SCAFFOLD_PATH)
            path = path.replace(IUNO_BASE_PATH, SCAFFOLD_TEMP_PATH)
            tempPath = path
            if not os.path.exists(path):
                os.mkdir(path)
        
        if not name:
            print("Please type project name.")
            return

        if not path:
            path = os.path.join(os.getcwd(), name)
        else:
            path = os.path.join(path, name)

        # FIXME: 判斷 Manage.py
        if os.path.abspath(path) == os.path.abspath(
                os.path.join(os.getcwd(), name)):
            os.system(
                "%s manage.py startproject %s" % (
                    sys.executable, name))
        else:
            if not os.path.exists(path):
                os.mkdir(path)

            os.system(
                "%s manage.py startproject %s \"%s\"" % (
                    sys.executable, name, path))

        

        #if not (
        #    django.VERSION[0] == 2 and 
        #    django.VERSION[1] == 2 and 
        #    django.VERSION[2] == 10):
        #    raise CommandError("Only support django 2.2.10")

        settingPath = os.path.join(path, name, 'settings.py')
        urlPath = os.path.join(path, name, 'urls.py')

        def renderFile(template, path, fileName, linux=False):
            temp = get_template(template)
            
            context = {
                "name": name,
                "useConda": False,
                "version": IUNO_VERSION,
                "envName": os.environ.get("CONDA_DEFAULT_ENV", "%s_ENV" % name),
            }
                
            content = temp.render(context)

            if linux:
                with open(os.path.join(path, fileName), 'wb') as f:
                    if not six.PY3:
                        f.write(six.b(content.replace('\r\n', '\n')))
                    else:
                        f.write(content.replace('\r\n', '\n').encode())
            else:
                with open(os.path.join(path, fileName), 'w') as f:
                    f.write(content)

        # file -----------------------------------------------------------------
        renderFile('scaffold/Activate.bat', path, 'ActivateEnv.bat')
        renderFile('scaffold/Configure.bat', path, 'Configure.bat')
        renderFile('scaffold/Configure.sh', path, 'Configure.sh', linux=True)
        renderFile('scaffold/README.txt', path, 'README.txt')
        renderFile('scaffold/REQUIREMENTS.txt', path, 'REQUIREMENTS.txt')
        
        managementPath = os.path.join(
            os.path.dirname(settingPath), "management")
        if not os.path.exists(managementPath):
            os.mkdir(managementPath)
            
        with open(os.path.join(managementPath, "__init__.py"), 'w') as f:
            pass
            
        commandPath = os.path.join(managementPath, "commands")
        if not os.path.exists(commandPath):
            os.mkdir(commandPath)
            
        with open(os.path.join(commandPath, "__init__.py"), 'w') as f:
            pass
            
        # 之所以 template 用 .txt 是因為 setup 的時候會嘗試去 import
        renderFile('scaffold/Deploy.txt', commandPath, 'deploy.py')

        # Settings.txt ----------------------------------------------------------
        with open(settingPath, 'r') as setting:
            content = setting.read()

        # Header
        context = {
            "year": datetime.date.today().year,
        }
        temp = get_template('scaffold/HeaderTemplate.txt')
        header = temp.render(context)
        content = header + content

        # debug & mode
        context = {
            "name": name.lower(),
            "version": IUNO_VERSION,
            "pandora": pandora,
        }
        temp = get_template('scaffold/DebugTemplate.txt')
        debug = temp.render(context)
        content = content.replace('DEBUG = True', debug)

        # allow host
        temp = get_template('scaffold/AllowHostTemplate.txt')
        host = temp.render(context)
        content = content.replace('ALLOWED_HOSTS = []', host)

        # install app
        context = {
            "name": name,
        }
        temp1 = get_template('scaffold/InstalledApps.txt').render()
        temp2 = get_template(
            'scaffold/InstalledAppsTemplate.txt').render(context)
        content = content.replace(temp1, temp2)

        # db & static
        temp = get_template('scaffold/Database.txt')
        database = temp.render()
        content = content.replace(database, '')

        temp = get_template('scaffold/Static.txt')
        static = temp.render()
        content = content.replace(static, '')

        temp = get_template('scaffold/TimeZone.txt')
        timeZone = temp.render()
        content = content.replace(timeZone, '')
        
        # template & middleware & auth password
        temp = get_template('scaffold/Templates.txt')
        templates = temp.render()
        content = content.replace(templates, '')
        
        temp = get_template('scaffold/Middleware.txt')
        middleware = temp.render()
        content = content.replace(middleware, '')
        
        temp = get_template('scaffold/Auth.txt')
        auth = temp.render()
        content = content.replace(auth, '')
        
        # wsgi & root urlconf
        temp = get_template('scaffold/Wsgi.txt')
        wsgi = temp.render(context)
        content = content.replace(wsgi, '')
        
        temp = get_template('scaffold/Urlconf.txt')
        urlconf = temp.render(context)
        content = content.replace(urlconf, '')
        
        # default auto field # TODO test if removing auto field is fine
        temp = get_template('scaffold/DefaultAutoField.txt')
        default_auto_field = temp.render(context)
        content = content.replace(default_auto_field, '')
        
        # Footer
        temp = get_template('scaffold/FooterTemplate.txt')
        footer = temp.render()
        content = content.strip() + '\n' + footer

        with open(settingPath, 'wb') as setting:
            if not six.PY3:
                setting.write(six.b(utf8(content.replace('\r\n', '\n'))))
            else:
                setting.write(content.replace('\r\n', '\n').encode())
        # Urls.txt --------------------------------------------------------------
        with open(urlPath, 'r') as url:
            content = url.read()

        context = {
            "name": name.lower(),
            "version": IUNO_VERSION,
            "pandora": pandora,
        }
        
        temp1 = get_template('scaffold/Urls.txt').render()
        temp2 = get_template('scaffold/UrlsTemplate.txt').render(context)
        content = content.replace(temp1, temp2)

        with open(urlPath, 'w') as url:
            url.write(content)
            
        # pandora
        if pandora:
            copyTree(path, pandoraPath)
            shutil.rmtree(tempPath)
            



